@@ -10,38 +10,50 @@ use Net::Autoconfig::Template;
 use Data::Dumper;
 use POSIX ":sys_wait_h";
 use Cwd;
-use version; our $VERSION = version->new("v1.13.2");
 
-################################################################################
+our $VERSION = '1.01';
+our $PATH    = getcwd();
+
 # Constants and Global Variables
 ################################################################################
 
-use constant TRUE	=>	1;
-use constant FALSE	=>	0;
+# Initialize logging
+{
+    my $filename  = join("/",$PATH,"logging.conf");
+    if (-e $filename)
+    {
+        eval
+        {
+            Log::Log4perl::init($filename);
+        };
+        if ($@) {
+            print STDERR "Failed to initialize '$filename' even though it exists.\n";
+            print STDERR "Logging to STDERR.";
+            Log::Log4perl->easy_init($WARN);
+        }
+    }
+    else
+    {
+        print STDERR "logging.conf does not exist!";
+        print STDERR "Logging to STDERR.";
+        Log::Log4perl->easy_init($INFO);
+    }
+}
+
+
+use constant TRUE   =>  1;
+use constant FALSE  =>  0;
 
 use constant MAXIMUM_MAX_CHILDREN => 256; # Absolute Maximum # of child processes (if using bulk mode)
 use constant DEFAULT_MAX_CHILDREN => 64;  # Default max # of child processes (if using bulk mode)
 use constant MINIMUM_MAX_CHILDREN => 1;   # Absolute Minimum # of child processes (if using bulk mode)
 
-use constant DEFAULT_DIR          => '/usr/local/etc/autoconfig';
-use constant DEFAULT_LOGFILE      => DEFAULT_DIR . '/logging.conf';
-
 use constant MAXIMUM_LOG_LEVEL    => 5;   # Absolute Maximum log level
 use constant DEFAULT_LOG_LEVEL    => 3;   # Set the default log level to info
 use constant MINIMUM_LOG_LEVEL    => 0;   # Absolute Minimum log level
 
 use constant DEFAULT_BULK_MODE    => TRUE; # Enable parallel processing by default
 
-####################
-# Friendly User Prompt Messages
-####################
-use constant USER_PROMPTS => {
-    'password'          =>  "Device Access Password",
-    'enable_password'   =>  "Device Admin  Password",
-    'console_password'  =>  "Console Server Access Password",
-};
-
-
 # A hash ref to store child processes.
 # Contains active processes, and return values
 our $CHILD_PROCESSES = {};
@@ -82,70 +94,19 @@ $SIG{'CHLD'} = sub { $ZOMBIES++ };
 #
 ########################################
 sub new {
-	my $invocant  = shift; # calling class	
-	my $class     = ref($invocant) || $invocant;
-	my $log       = Log::Log4perl->get_logger($class);
-    my %user_data = @_;
-
-	my $self = {
-				bulk_mode		=>	DEFAULT_BULK_MODE,
-				log_level		=>	DEFAULT_LOG_LEVEL,
-				max_children	=>	DEFAULT_MAX_CHILDREN,
-                logfile         =>  DEFAULT_LOGFILE,
-				};
-
-    $self = bless $self, $class;
-
-    $self->logfile(      $user_data{'logfile'} );
-    $self->init_logging();
-
-    $self->bulk_mode(    $user_data{'bulk_mode'} );
-    $self->log_level(    $user_data{'log_level'} );
-    $self->max_children( $user_data{'max_children'} );
-
-	$log->info("########################################");
-	$log->info("#       Net::Autoconfig Started        #");
-	$log->info("########################################");
-	return $self;
-}
-
-########################################
-# init_logging
-# public method
-#
-# Initialize logging for Net::Autoconfig.
-# If multiple Net::Autoconfig objects
-# are created, calling this will affect
-# all of them; it changes their logging
-# definitions.
-#
-# Returns undef
-########################################
-sub init_logging {
-    my $self = shift;
-
-    # XXX - Setup a saner, \$log_string
-    # config so it's not just to stdout/stderr
-
-    if ( -e $self->logfile )
-    {
-		eval {
-			Log::Log4perl::init( $self->logfile );
-		};
-		if ($@) {
-			print STDERR "Failed to initialize '" . $self->logfile
-                          . "' even though it exists.\n";
-			print STDERR "Logging to STDERR.";
-			Log::Log4perl->easy_init($WARN);
-		}
-    }
-    else
-    {
-		print STDERR "logging.conf does not exist!";
-		print STDERR "Logging to STDERR.";
-		Log::Log4perl->easy_init($INFO);
-    }
-    return;
+    my $invocant = shift; # calling class   
+    my $class    = ref($invocant) || $invocant;
+    my $log      = Log::Log4perl->get_logger("Net::Autoconfig");
+    my $self = {
+                bulk_mode       =>  DEFAULT_BULK_MODE,
+                log_level       =>  DEFAULT_LOG_LEVEL,
+                max_children    =>  DEFAULT_MAX_CHILDREN,
+                };
+    
+    $log->info("########################################");
+    $log->info("#       Net::Autoconfig Started        #");
+    $log->info("########################################");
+    return bless $self, $class;
 }
 
 ########################################
@@ -162,7 +123,7 @@ sub init_logging {
 sub bulk_mode {
     my $self = shift;
     my $mode = shift;
-    my $log  = Log::Log4perl->get_logger( ref($self) );
+    my $log  = Log::Log4perl->get_logger("Net::Autoconfig");
 
     if (defined $mode)
     {
@@ -186,7 +147,7 @@ sub bulk_mode {
 sub log_level {
     my $self  = shift;
     my $level = shift;
-    my $log   = Log::Log4perl->get_logger( ref($self) );
+    my $log   = Log::Log4perl->get_logger("Net::Autoconfig");
     
     if (defined $level)
     {
@@ -222,7 +183,7 @@ sub log_level {
 sub max_children {
     my $self         = shift;
     my $max_children = shift;
-    my $log          = Log::Log4perl->get_logger( ref($self) );
+    my $log          = Log::Log4perl->get_logger("Net::Autoconfig");
 
     if (defined $max_children)
     {
@@ -275,94 +236,31 @@ sub get_report {
     return wantarray ? %$report : $report;
 }
 
-########################################
-# logfile
-# public method
-#
-# Accessor/Mutator
-# 
-####################
-# Mutator
-#
-# Sets the logfile if it exists
-# (assumes current working directory if
-# the filename is not specified absolutely.)
-#
-# Else, it sets the logfile to the default.
-#
-# Returns:
-#   Success =>  undef
-#   Failure =>  error message
-####################
-# Accessor
-#
-# Returns
-#   the logfile's absolute path
-########################################
-sub logfile {
-    my $self          = shift;
-    my $logfile       = shift;
-    my $return_value;
-
-    if ( defined $logfile )
-    {
-        # Check for abs path
-        if ( $logfile !~ /^\// )
-        {
-            $logfile = join('/', getcwd(), $logfile);
-        }
-
-        if (-e $logfile)
-        {
-            $self->{'logfile'} = $logfile;
-        }
-        else
-        {
-            $self->{'logfile'} = DEFAULT_LOGFILE;
-            print STDERR "\n'$logfile' either does not exist or is unreadable\n";
-            print STDERR "\nUsing default logfile " . DEFAULT_LOGFILE . "\n";
-        }
-        undef $return_value;
-    }
-    else
-    {
-        $return_value = $self->{'logfile'};
-    }
-    return $return_value;
-}
-
 ########################################
 # load_devices
 # public method
 #
 # This method looks at a device config
 # file and returns
-# an array ref of Net::Autoconfig::Device's.
-# Devices are returned in the same order as
-# in the device file.
+# a hash ref of Net::Autoconfig::Device's.
+# Where the key = hostname
+# E.g. { hostname => Net::Autoconfig::Device }
 #
 # Returns:
-# array context     =>  an array of Devices
-# scalar context    =>  an array ref of Devices
+# array context     =>  a hash of Devices
+# scalar context    =>  a hash ref of Devices
 # undef             =>  failure
 ########################################
 sub load_devices {
     my $self = shift;
     my $filename = shift;
-    my $log      = Log::Log4perl->get_logger( ref($self) );
-    my $devices  = [];    # an array ref of Net::Autoconfig::Devices, key = hostname
+    my $log      = Log::Log4perl->get_logger("Net::Autoconfig");
+    my $devices  = {};    # an array ref of Net::Autoconfig::Devices, key = hostname
     my $file_format;      # indicates if the file is a hash of arrays, or as hash of hashes of arrays
     my $file_hash_depth;  # an integer of the number of levels of hashes in the device file
     my $current_device;   # the name of the current device to add parameters too
-    my $line_counter;     # The current line we're on in the device config file (used for logging)
-    my $default_device;   # The default device, helps populate new devices
     $filename or $filename = "";
-
-    # Check for abs path
-    if ( $filename !~ /\// )
-    {
-        $filename = join("/", getcwd(), $filename);
-    }
+    $filename = join("/", $PATH, $filename);
 
     (&_file_not_usable($filename, "device config")) and return;
 
@@ -377,89 +275,60 @@ sub load_devices {
     }
 
     # Create this here in case someone decides not to use the default
-    # device.  Re-set the auto-disover bit
-    $default_device = Net::Autoconfig::Device->new( 'auto_discover' => FALSE );
-    $default_device->hostname('autoconfig-default');
-
+    # device.
+    $devices->{default} = Net::Autoconfig::Device->new();
     while (my $line = <DEVICES>)
     {
         chomp $line;
         next if $line =~ /^#/;
         next if $line =~ /^\s*$/;
 
-        $line_counter++;
-
         if ($line =~ /^:/)
         {
             # some type of host declaration (host or default)
             $line =~ s/^://;
             $line =~ s/:$//;
-            $line =~ s/\s*(.*?)\s*/$1/; #remove preceding and trailing whitespace
-
-            if (not $line)
+            $current_device = $line;
+            if ($current_device =~ /default/)
             {
-                $log->warn("In device file, undef device '::' line at $line_counter.");
-                next;
-            }
-
-            if ($line =~ /default/i)
-            {
-                $current_device = $default_device;
+                # Allow redefinition of the default device.
+                $devices->{default} = Net::Autoconfig::Device->new();
+                $devices->{default}->set("auto_discover", TRUE);
             }
             elsif ($line =~ /^end$/)
             {
-                if ($current_device->hostname !~ /autoconfig-default/i)
-                {
-                    $log->trace("Adding " . $current_device->hostname
-                                . " to the list of devices.");
-                    $current_device = $current_device->auto_discover;
-                    push(@$devices, $current_device);
-                }
                 undef $current_device;
             }
             else
             {
-                $current_device = Net::Autoconfig::Device->new(
-                                                $default_device->get(),
-                                                'hostname' => $line,
+                if (not $current_device)
+                {
+                    $log->warn("No device configured!");
+                    next;
+                }
+                $devices->{$current_device} = Net::Autoconfig::Device->new(
+                                                %{ $devices->{default}->get() },
+                                                'hostname' => $current_device,
                                                 );
             }
         }
-        elsif ($line =~ /\s*(.*?)\s*=\s*(.*?)\s*$/)
+        elsif ($line =~ /\s*(\w+)\s*=\s*(.*?)\s*$/)
         {
             my $key = $1;
             my $value = $2;
             if (not $current_device)
             {
-                $log->warn("No device is currently configured at line $line_counter.");
+                $log->warn("No device is currently configured!  Line = '$line'.");
                 next;
             }
             if ($log->is_trace())
             {
-                $log->trace("line # = $line_counter");
-                $log->trace("line   = '$line'");
-                $log->trace("key    = '$key'");
-                $log->trace("value  = '$value'");
+                $log->trace("line = '$line'");
+                $log->trace("key = '$key'");
+                $log->trace("value = '$value'");
             }
 
-            if ($value =~ /\<prompt\>/i)
-            {
-                my $user_prompts = USER_PROMPTS;
-                my $hostname     = $current_device->hostname;
-                my $message      = $user_prompts->{$key};
-                if (not $message)
-                {
-                    $message = $key;
-                }
-                if ($hostname eq $default_device->hostname)
-                {
-                    $hostname = "Default";
-                }
-                $message = "[$hostname] - $message";
-                $value = &_get_password($message);
-            }
-
-            $current_device->set($key => $value);
+            $devices->{$current_device}->set($key => $value);
         }
         else
         {
@@ -467,8 +336,10 @@ sub load_devices {
         }
     }
 
+    delete $devices->{default};
+
     close(DEVICES);
-    return wantarray ? @$devices : $devices;
+    return wantarray ? %$devices : $devices;
 }
 
 ########################################
@@ -476,31 +347,27 @@ sub load_devices {
 # public method
 #
 # Load a configuration template from disk.
-# These files use the colon file format.
+# These files use the colon-operator format.
 # See documentation for more details.
 #
-# Returns
-#   success =>  a hash ref of the different hosts/devices types
-#   failure =>  undef
+# Returns:
+#   array context   =>  a hash of the different hosts/devices types
+#   scalar context  =>  a hash ref of the different hosts/devices types
+#   failure         => undef
 ########################################
 sub load_template {
-    my $self     = shift;
+    my $self = shift;
     my $filename = shift;
-    my $log      = Log::Log4perl->get_logger( ref($self) );
+    my $log = Log::Log4perl->get_logger('Net::Autoconfig');
     my $template;
     $filename or $filename = "";
-
-    # Check for abs path
-    if ( $filename !~ /^\// )
-    {
-        $filename = join("/", getcwd(), $filename);
-    }
+    $filename = join("/", $PATH, $filename);
 
     (&_file_not_usable($filename, "template file")) and return;
 
     $template = Net::Autoconfig::Template->new($filename);
 
-    return $template;
+    return wantarray ? %{ $template } : $template;
 }
 
 ########################################
@@ -531,7 +398,7 @@ sub autoconfig {
     my $devices = shift;
     my $template = shift;
     my $failed_ping_test;   # results from doing the ping test on the device
-    my $log = Log::Log4perl->get_logger( ref($self) );
+    my $log = Log::Log4perl->get_logger("Net::Autoconfig");
 
     if (ref($self) !~ /Net::Autoconfig/)
     {
@@ -545,10 +412,10 @@ sub autoconfig {
         return "No devices passed to autoconfig.";
     }
     
-    if (not ref ($devices) eq "ARRAY")
+    if (not ref ($devices) eq "HASH")
     {
-        $log->warn("Devices were not passed as an array ref.");
-        return "Devices were not passed as an array ref.";
+        $log->warn("Devices were not passed as a hash ref.");
+        return "Devices were not passed as a hash ref.";
     }
 
     if (not $template)
@@ -557,8 +424,9 @@ sub autoconfig {
         return "No template passed to autoconfig.";
     }
 
-    foreach my $device ( @$devices )
+    foreach my $device_key (keys %$devices)
     {
+        my $device       = $devices->{$device_key};
         my $child_pid;   # PID of the child process (if used)
 
         if ($log->is_trace)
@@ -623,7 +491,7 @@ sub autoconfig {
 
         if ($failed_ping_test)
         {
-            my $hostname = $device->hostname || "";
+            my $hostname = $device->hostname || $device_key;
             $log->warn("$hostname was not reachable via ping.  Aborting configuration attempt.");
             if ($self->bulk_mode)
             {
@@ -641,8 +509,6 @@ sub autoconfig {
         $device->get_admin_rights();
         $device->disable_paging();
 
-        $device->provision and $device->lookup_model;
-
         # Do the generic, device model/type template first
         # device->model returns an array ref, a device can match more
         # than one device type.  Take the first one that exists in the template.
@@ -701,49 +567,6 @@ sub autoconfig {
 ############################################################
 # Private Methods
 ############################################################
-#
-########################################
-# _get_password
-# private function
-#
-# Get a password from the user.
-# I.e. prevent local echoing of their
-# password.
-########################################
-sub _get_password {
-    my $prompt     = shift;
-    my $log        = Log::Log4perl->get_logger( __PACKAGE__ );
-    my $message    = "";
-    my $user_input = "";
-
-    if (not $prompt)
-    {
-        $log->debug("get_password - Prompt not specified");
-        $prompt = "(Not Specified)";
-    }
-
-    $message = "[User Input] - $prompt: ";
-
-    $log->trace("get_password - Message = '$message'");
-    print $message;
-
-    # Hide user input
-    # This only works on linux/unix machines.
-    $log->trace("get_password - hiding user text input");
-    system("stty -echo");
-
-    $user_input = <STDIN>;
-    chomp($user_input);
-    $log->trace("get_password - password = '$user_input'");
-
-    # Show user input again.
-    $log->trace("get_password - showing user text input");
-    system("stty echo");
-    print "\n";
-
-    return $user_input;
-}
-
 
 ########################################
 # _file_not_usable
@@ -759,10 +582,10 @@ sub _get_password {
 # Return TRUE if it's not
 ########################################
 sub _file_not_usable {
-    my $filename     = shift;
+    my $filename = shift;
     my $file_descrip = shift;
     my $working_dir  = getcwd();
-    my $log          = Log::Log4perl->get_logger( __PACKAGE__ );
+    my $log = Log::Log4perl->get_logger("Net::Autoconfig");
     $file_descrip = $file_descrip || "";
 
     if (! $filename)
@@ -810,7 +633,7 @@ sub _failed_ping_test {
 ##############################
 sub _reaper {
     my $zombie;
-    my $log = Log::Log4perl->get_logger( __PACKAGE__ );
+    my $log = Log::Log4perl->get_logger("Net::Autoconfig");
 
     $log->trace("Start reaping zombies.");
     $log->trace("Number of zombies        : $ZOMBIES");
@@ -954,23 +777,13 @@ you want, but only the following ones will have an effect.
  log_level
  max_children
 
-=item init_logging()
-
-Initialize logging for the module.  Right now, it loads
-the default config file.
-
 =item load_devices("filename")
 
-Return an array ref of all of the devices in the
+Return a hash ref of all of the devices in the
 specified file.  The file needs to be in colon format.  You can specify
 any key => value pair that you want.  There are some predefined ones,
 but you can safely ignore those if you want.
 
-Devices are returned in the same order as they appear in the
-device configuration file.
-
-B<Note:> Versions prior to 1.12 returned a hash ref.
-
 See the documenation on B<Net::Autoconfig::Device> for information
 regarding the fileformat.
 
@@ -987,18 +800,14 @@ own defaults.
 See the documenation on B<Net::Autoconfig::Template> for information
 regarding the fileformat.
 
-Returns a Net::Autoconfig::Template device.
-
 =item autoconfig($devices, $template)
 
-Takes an array ref of devices and a Net::Autoconfig::Template object.  It
+Takes a hash of devices and a Net::Autoconfig::Template object.  It
 will try to configure a general per-class-type set of commands first
 and then a set of commands specific to that device.  Any combination
 of command sets are permissable.  It will send a notification if
 any commands fail for a paticular device.
 
-B<Note:> Prior to version 1.12, $devices was a hash ref
-
 =item bulk_mode(TRUE/FALSE/undef)
 
 Bulk mode is enabled by default.
@@ -1009,7 +818,7 @@ bulk_mode status if passed nothing (i.e. undef).
 
 =item max_children($value/undef)
 
-Max_children is 64 by default.
+Max_children is 128 by default.
 
 If passed undef, it returns the current maximum number of
 simultaneous processes to run if bulk_mode is enabled.
@@ -1018,19 +827,9 @@ to that value.  The absolute maximum number of child processes
 is 256.  If that is too few, you can modify that value in the
 module.
 
-=item logfile($filename/undef)
-
-Access/Mutator method.
-
-Sets the location of the logfile if passed the filename.
-
-Returns the location of the logfile if passed undef.
-
 =item log_level($level/undef)
 
-*Note: This value currently has no effect on the operation
-of this module.
-
+This value has no effect on the operation of this module.
 Set the log level using the logging.conf file.
 
 If passed a value, it will set the log level to that value.
