@@ -7,7 +7,8 @@ use warnings;
 use base "Net::Autoconfig::Device";
 use Log::Log4perl qw(:levels);
 use Expect;
-use version; our $VERSION = version->new('v1.2.2');
+
+our $VERSION = '1.00';
 
 #################################################################################
 ## Constants and Global Variables
@@ -32,7 +33,7 @@ use constant SHORT_TIMEOUT  =>  5;
 # Yay!
 ####################
 my $expect_ssh_key_cmd   = '[
-                            -re => "continue connecting",
+                            -re => "continue connecting \(yes\/no\)",
                             sub
                             {
                                 $session->clear_accum();
@@ -58,8 +59,7 @@ my $expect_password_cmd = '[
                             }
                         ]';
 my $expect_hp_continue_cmd = '[
-                            #-re => "any key to continue",
-                            -re => "continue",
+                            -re => "any key to continue",
                             sub
                             {
                                 $session->clear_accum();
@@ -213,7 +213,7 @@ sub connect {
     if ($self->access_method =~ /^ssh$/)
     {
         # Force using ssh version 1 due to old firmware problems.
-        $spawn_cmd = join(" ", $self->access_cmd, "-l", $self->username, $self->hostname);
+        $spawn_cmd = join(" ", $self->access_cmd, "-1 -l", $self->username, $self->hostname);
     }
     else
     {
@@ -265,7 +265,6 @@ sub connect {
     # accept the ssh key
     # send the username
     # send the password
-    # bypass the initial login screen
     # verify connection (exec or priv exec mode)
     ####################
     push(@expect_commands, [
@@ -277,23 +276,23 @@ sub connect {
                             eval $expect_exec_mode_cmd,
                             eval $expect_priv_mode_cmd,
                     ]);
+    push(@expect_commands, [
+                            # Get past the initial login banner
+                            eval $expect_hp_continue_cmd,
+
+                            eval $expect_username_cmd,
+                            eval $expect_password_cmd,
+                            eval $expect_exec_mode_cmd,
+                            eval $expect_priv_mode_cmd,
+                    ]);
     push(@expect_commands, [
                             eval $expect_username_cmd,
                             eval $expect_password_cmd,
-                            # Get past the initial login banner
-                            eval $expect_hp_continue_cmd,
                             eval $expect_exec_mode_cmd,
                             eval $expect_priv_mode_cmd,
                     ]);
     push(@expect_commands, [
                             eval $expect_password_cmd,
-                            # Get past the initial login banner
-                            eval $expect_hp_continue_cmd,
-                            eval $expect_exec_mode_cmd,
-                            eval $expect_priv_mode_cmd,
-                    ]);
-    push(@expect_commands, [
-                            eval $expect_hp_continue_cmd,
                             eval $expect_exec_mode_cmd,
                             eval $expect_priv_mode_cmd,
                     ]);
